COMMENT ^
       Bigfoot Init v1.3 source code in TASM's Ideal mode.
       Works OK with TASM v4 and TLINK v3.

       Some Quantum Bigfoot 2550A IDE drives produce read errors
       when data is read very slowly, eg. when uploading a file
       with a modem. BFTest is intended to check your hard disk
       for that stupid bug.

       BFInit writes the data file, which is then read by BFTest.
       Use them at your own risk(?).

       (C) Copyright 1996 by Sami Farin <sfarin@ratol.fi>
       All Rights Reserved. Freeware.

        ^

.Model Tiny
.386
.Code

Ideal
Org 100h

Include "Macros.INC"

Start:
       Cld
       Push    0B800h
       Pop     GS

       Disp    StartText

       Mov     AX, SS
       Add     AX, 1000h                ; Otherwise we'd have 576 B of stack
       Mov     SS, AX

       Mov     AX, 3D02h
       Xor     CX, CX                   ; Perhaps not needed
       Lea     DX, [FName]
       Int     21h                      ; Open it
       Jnc     NoCreate                 ; Dont's truncate it

       Mov     AH, 3Ch                  ; Create it
       Int     21h

NoCreate:
       Mov     [NullH], AX              ; File handle
       Xchg    AX, BX
       Mov     AX, 4202h
       Cwd                              ; CX:DX=0
       Int     21h                      ; Move pointer to end of file
       Push    DX AX
       Pop     EAX
       Mov     [CurrPos], EAX           ; CurrPos contains file size (32 bits)
       Shr     EAX, 9                   ; Add right nr of sectors
       Mov     [Sects], EAX             ; Sects=filesize/512
       Call    InitWriteBuffer
       Jmp     SkipFirst

MoreSpace:
       Call    InitData                 ; Initialize the buffer

SkipFirst:
       Call    DispPos                  ; Display bytes processed

       CheckKey
       Jnz     Bye

       Mov     AH, 40h
       Mov     BX, [NullH]
       Mov     CX, 64000                ; Big buffers --> less overhead
       Lea     DX, [FillArea2]
       Int     21h

       MovZx   EAX, AX
       Add     [CurrPos], EAX
       Xor     AX, CX                   ; Or Cmp or Sub or And or Test ...
       Jz      MoreSpace                ; Loop if bytes written is 64000

       Sub     [Sects], 125             ; Disk space available less than 125
                                        ; sectors, this is the fixup

LittleSpace:
       Mov     BP, 512/512              ; We're processing one sector per loop
       Call    InitData2
       Call    DispPos

       CheckKey
       Jnz     Bye

       Mov     AH, 40h
       Mov     BX, [NullH]
       Mov     CX, 512
       Lea     DX, [FillArea2]
       Int     21h                      ; Write file

       MovZx   EAX, AX                  ; AX=bytes written
       Add     [CurrPos], EAX
       Test    AX, AX                   ; We have yet 512 or more bytes free
       Jnz     LittleSpace

       Jmp     SkipKey

Bye:
       WaitKey

SkipKey:
       Mov     AX, 4C00h
       Int     21h

DispPos:
       Mov     EAX, [Sects]
       Mov     CX, 9                    ; Displays 9.999.999 at max
       Mov     DI, (160*2) + (2*22)     ; Video mem coords (row=3, column=22)
       Call    DispBig2

       DB      66h, 0B8h
       CurrPos DD 0                     ; Mov EAX, [CurrPos]
       Mov     DI, (160*2) + (2*12)

DispBig:
       Mov     CL, 13                   ; 2^32-1 at max

DispBig2:
       Lea     EBX, [10]

LongDivLoop:
       Xor     EDX, EDX
       Div     EBX
       Add     DL, "0"
       Mov     DH, 0Eh                  ; Our nice colour (yellow)
       Mov     [GS:DI], DX
       Dec     DI
       Dec     DI

       Cmp     CX, 11
       Je      Dot1
       Cmp     CX, 7
       Je      Dot1
       Cmp     CX, 3
       Je      Dot1

       Dec     CX
       Jnz     LongDivLoop
       Ret

Dot1:
       Mov     [Word GS:DI], 0C2Eh      ; Our nice little dot
       Dec     DI
       Dec     DI
       Dec     CX
       Dec     CX
       Jmp     LongDivLoop

InitWriteBuffer:                        ; Initializes buffer to be written
       Mov     BP, 64000/512            ; Process 64000 bytes
       Lea     DI, [FillArea2]          ; Buffer starts here

@Loop:
       Lea     SI, [FillArea]           ; Our temporary processing area
       Inc     [Sects]
       Push    DI
       Call    PutSect                  ; Put the values
       Pop     DI
       Mov     CX, CopyCount
       Rep     MovsB                    ; Copy to new buffer
       Mov     CX, (512-CopyCount) SHR 2 - 1 ; How nice
       Mov     EAX, "tinI"
       Rep     StosD                    ; Init buffer
       Mov     EAX, 0A0D0A0Dh           ; Two linefeeds
       StosD
       Dec     BP
       Jnz     @Loop
       Ret

InitData:                               ; Put new values to the buffer
       Mov     BP, 64000/512            ; Processing 512 bytes per one loop

InitData2:
       Lea     DI, [FillArea2+(SectEnd-FillArea-1)]

InitLoop:
       Inc     [Sects]
       Mov     EAX, [Sects]
       Mov     [DI+19], EAX             ; Put it in the right place
       Call    PutSect2
       Add     DI, 521                  ; Next sector
       Dec     BP
       Jnz     InitLoop
       Ret

PutSect:
       Lea     DI, [SectEnd-1]

PutSect2:
       Mov     CX, 9
       Mov     EAX, [Sects]
       Lea     EBX, [10]

LongDivLoop2:
       Xor     EDX, EDX
       Div     EBX
       Add     DL, "0"
       Mov     [DI], DL
       Dec     DI
       Cmp     CX, 7
       Je      Dot2
       Cmp     CX, 3
       Je      Dot2

Enddi:
       Dec     CX
       Jnz     LongDivLoop2
       Ret

Dot2:
       Dec     DI
       Dec     CX
       Dec     CX
       Jmp     LongDivLoop2

StartText      DB "BFInit v1.3 started.$"
FName          DB "BFRead$$.$$$",0
NullH          DW 0

FillArea       DB "This is sector "
SectStart      DB "x.xxx.xxx"
SectEnd        DB " of BFRead$$.$$$. "
Sects          DD 0
               DB 13,10
CopyCount      =  $-FillArea

Align 4
FillArea2:
CopyR          DB "(C) Copyright 1996 by Sami Farin <sfarin@ratol.fi>. "
               DB "All Rights Reserved. Freeware."

END Start
