COMMENT ^
       Bigfoot Test v1.3 source code in TASM's Ideal mode.
       Works OK with TASM v4 and TLINK v3.

       Some Quantum Bigfoot 2550A IDE drives produce read errors
       when data is read very slowly, eg. when uploading a file
       with a modem. BFTest is intended to check your hard disk
       for that stupid bug.

       BFInit writes the data file, which is then read by BFTest.
       Use them at your own risk(?).

       (C) Copyright 1996 by Sami Farin <sfarin@ratol.fi>
       All Rights Reserved. Freeware.

        ^

.Model Tiny
.386
.Code

Ideal
Org 100h

Include "Macros.INC"

Start:
       Push    0B800h
       Pop     GS

       Mov     AX, 3D02h
       Lea     DX, [FName]
       Int     21h                      ; Open file for reading

       Jnc     NoError

       Disp    FFound                   ; ...not found
       Ret				; Exit to DOS

NoError:
       Call    Init

       Mov     SI, 80h
       LodsB
       Cmp     AL, 1
       Jbe     ChkDir

Options:
       LodsB
       Cmp     AL, " "
       Je      Options
       Cmp     AL, 0Dh
       Je      ChkDir
       Cmp     AL, "0"
       Jb      Options
       Cmp     AL, "9"
       Jbe     PutTicks                 ; Jump if between 1-9
       And     AL, 11011111b
       Cmp     AL, "Z"
       Ja      Options
       Cmp     AL, "A"
       Jb      Options

       Mov     [OutputDrv], AL

       Call    CheckDir
       Jnc     Options

       Disp    ErrorDrive               ; Failed to write to specified drive

ErrExit:
       Mov     AX, 4C01h
       Int     21h

PutTicks:
       Sub     AL, "0"
       Jnz     NoFast
       Mov     [Byte FastOpt], 0C3h     ; Ret instead of Nop

NoFast:
       Mov     [Ticks], AL              ; Wait this many ticks
       Jmp     Options                  ; 1/(4.77M/4/65536) seconds ~ 0.05496s

CheckDir:
       Mov     AH, 3Ch
       Xor     CX, CX
       Lea     DX, [OutputDrv]
       Int     21h                      ; Create file on specified drive
       Jnc     ItsThere

       Mov     [Byte ErrFile-1], 0      ; Dir must end with a null
       Mov     AH, 39h
       Lea     DX, [OutputDrv]
       Int     21h                      ; Create directory
       Mov     [Byte ErrFile-1], "\"
       Jnc     DirMade
       Ret

ItsThere:
       Xchg    AX, BX
       Mov     AH, 3Eh
       Int     21h                      ; Close that file

       Mov     AH, 41h
       Int     21h                      ; And remove it

DirMade:
       Mov     [DirOK], DH
       Ret                              ; Carry not set (if no errors)

ChkDir:
       Cmp     [DirOK], 0               ; Output dir specified?
       Jne     DispWorkingDir
       Call    CheckDir
       Jnc     DispWorkingDir

       Disp    ErrorDrive
       Jmp     ErrorExit

DispWorkingDir:
       Mov     AH, 40h
       Mov     BX, 1                    ; Write to StdOut
       Mov     CX, StrL
       Lea     DX, [StartText]
       Int     21h

       Disp    InfText

CheckLoop:
       Call    DispPos

       Call    Wait_A_Bit               ; Some delay to confuse Bigfoot

       CheckKey
       Jnz     Bye

       Mov     [DWord GS:(160*3)], 1E520000h ; " R "
       Mov     [Word GS:(160*3)+4], 0

       Mov     AH, 3Fh
       Mov     BX, BP
       Mov     CX, 512
       Lea     DX, [FillArea2]
       Int     21h                      ; Read 512 bytes

       Test    AX, AX
       Jz      SkipKey			; EOF encountered before

       MovZx   EAX, AX
       Add     [CurrPos], EAX
       Mov     [DWord GS:(160*3)], 00001F43h ; "C  "

       Inc     [CurrS]                  ; Increase sector count
       Call    CheckSect
       Jz      CheckLoop                ; ZF set if sector OK

       Call    StoreSect                ; Save sector for later viewing
       Jmp     CheckLoop

Bye:
       WaitKey

SkipKey:
       Mov     AX, 4C00h
       Int     21h                      ; Bye bye

Init:
       Xchg    AX, BP                   ; From now on, BP contains the
       Cld                              ; file handle

       ; Disk caches must be flushed, since we aren't testing the memory!!!

       Mov     DI, 255                  ; In case you have 253 drives

FlushWin:                               ; Win95 reset drive function
       Mov     AX, 710Dh                ; Reset drive
       Mov     CX, 1                    ; Flush filesystem cache & buffers
       Mov     DX, DI                   ; Drive number
       Int     21h
       Dec     DI
       Cmp     DI, 2
       Jne     FlushWin

       Mov     AX, 4A10h
       Mov     BX, 0001h
       Int     2Fh                      ; Flush Smartdrive (or compatible)

       Mov     AX, 4A10h
       Mov     BX, 0002h
       Int     2Fh                      ; Reset Smartdrive's cache

       Mov     AX, 3                    ; clrscr();
       Int     10h

       Mov     AH, 19h
       Int     21h                      ; AL=current working drive
       Add     AL, "A"                  ; Make it a drive letter
       Mov     [OutputDrv], AL

       Ret

Wait_A_Bit:
FastOpt:
       Nop                             ; Or Ret

;       Cli                            ; To not to confuse DOS time...
;       Mov     AL, 00110100b          ; Set up to load initial timer count
;       Out     43h, AL
       Xor     EAX, EAX
       Mov     FS, AX
;       Out     40h, AL                ; Load count Lsb
;       Out     40h, AL                ; Load count Msb
;       Sti

WaitDay:
       Mov     EBX, [FS:46Ch]           ; Timer ticks since midnight
       Cmp     EBX, 1800B0h-11          ; Max. 1800B0h
       Ja      WaitDay                  ; Next-day-fix
                                        ; (...could have done a new
                                        ; interrupt routine)
       Mov     [DWord GS:(160*3)], EAX  ; "  W"
       Mov     [Word GS:(160*3)+4], 1A57h

       DB      66h, 83h, 0C3h           ; Add EBX, Ticks
Ticks  DB      2

WaitLoop:
       CheckKey
       Jnz     Bye
       Mov     EAX, [FS:46Ch]
       Sub     EAX, EBX
       Js      WaitLoop                 ; Enough timer ticks elapsed?
       Ret

FullDisk:
       Disp    FullText

ErrorExit:
       Mov     AX, 4CFFh
       Int     21h                      ; Exit w/ error level 255

StoreSect:
       Inc     [ReadErr]                ; One more error...

       Mov     AL, [Chr1]               ; Increase filename
       Inc     AX                       ; aaa --> aab
       Mov     [Chr1], AL
       Cmp     AL, "Z"+1
       Jne     NoOver

       Mov     [Chr1], "A"
       Mov     AL, [Chr2]
       Inc     AX
       Mov     [Chr2], AL
       Cmp     AL, "Z"+1
       Jne     NoOver

       Mov     [Chr2], "A"
       Mov     AL, [Chr3]
       Inc     AX
       Mov     [Chr3], AL
       Cmp     AL, "Z"+1
       Jne     NoOver

       Disp    EnoughNames              ; 17576 read errors (enough???)
       Jmp     ErrorExit

NoOver:
       Mov     SI, (160*3) + 8
       Lea     DI, [FillArea]           ; Grab current file pos. and sect
       Mov     CX, 13                   ; from video mem :)

NumbStore1:
       SegGs
       LodsW
       StosB
       Loop    NumbStore1

       LodsW                            ; si+=2
       Add     DI, 11
       Mov     CL, 9

NumbStore2:
       SegGs
       LodsW
       StosB
       Loop    NumbStore2

       Mov     AH, 3Ch			; Create and open errorfile
       Xor     CX, CX                   ; without attributes
       Lea     DX, [OutputDrv]
       Int     21h

       Xchg    AX, BX
       Mov     AH, 40h
       Mov     CX, 512 + SaveL
       Lea     DX, [SaveStart]
       Int     21h                      ; Write it

       Test    AX, AX
       Jz      FullDisk                 ; Out of space

       Mov     AH, 3Eh                  ; Close file (so that we don't
       Int     21h                      ; run out of file handles)

       DB      0BFh			; Mov DI, [DiPoint]
       DiPoint DW 160*4
       Mov     [DWord GS:DI], 0CE21h    ; "!" on red bg, blinking + one null
       Cmp     DI, 160*21               ; Time to start from the beginning?
       Jne     Skip
       Mov     [DiPoint], 160*4-2

Skip:
       Add     [DiPoint], 2
       Ret

DispPos:
       DB      66h, 0B8h                ; Mov EAX, [CurrS]
       CurrS   DD 0
       Inc     EAX
       Mov     DI, (160*3) + (2*26)
       Mov     CX, 9
       Call    DispBig2                 ; Display sectors read

       DB      66h, 0B8h                ; Mov EAX, [ReadErr]
       ReadErr DD 0
       Mov     DI, (160*3) + (2*36)
       Mov     CL, 9
       Call    DispBig2                 ; Display read errors (in sectors)

       DB      66h, 0B8h
       CurrPos DD 0			; Mov EAX, [CurrPos]
       Mov     DI, (160*3) + (2*16)

DispBig:
       Mov     CL, 13

DispBig2:
       Lea     EBX, [10]

LongDivLoop:
       Xor     EDX, EDX
       Div     EBX
       Add     DL, "0"
       Mov     DH, 0Eh                  ; Yellow
       Mov     [GS:DI], DX
       Dec     DI
       Dec     DI

       Cmp     CX, 11
       Je      Dot1
       Cmp     CX, 7
       Je      Dot1
       Cmp     CX, 3
       Je      Dot1

       Dec     CX
       Jnz     LongDivLoop
       Ret

Dot1:
       Mov     [Word GS:DI], 0C2Eh      ; Red .
       Dec     DI
       Dec     DI
       Dec     CX
       Dec     CX
       Jmp     LongDivLoop

CheckSect:
       Call    PutSect

       Lea     SI, [CompareText]
       Lea     DI, [FillArea2]
       Mov     CX, CompareL
       RepE    CmpsB                    ; Compare data for errors
       Jne     ReadError
       Mov     EAX, "tinI"
       Mov     CX, (512-CompareL) SHR 2 - 1
       RepE    ScasD
       Jne     Blaa
       Cmp     [DWord DI], 0A0D0A0Dh
       Jne     Blaa
       Ret                              ; Zero flag set

Blaa:
       Mov     [MispSign], "?"
       Xor     EAX, EAX
       Lea     DI, [MispEnd-1]
       Call    PutSect2
       Cmp     CL, 1                    ; CL=0, so Zero flag not set
       Ret

ReadError:                              ; Check if this is data from BFRead.$$$
       Lea     SI, [CompareText]
       Lea     DI, [FillArea2]
       Mov     CL, CompareL2
       RepE    CmpsB                    ; Check if "This is sector " is there
       Jne     Blaa
       Lea     DI, [FillArea2+CompareL] ; Skip some bytes and check the rest
       Mov     EAX, "tinI"
       Mov     CX, (512-CompareL) SHR 2 - 1
       RepE    ScasD
       Jne     Blaa
       Cmp     [DWord DI], 0A0D0A0Dh
       Jne     Blaa

       Mov     EAX, [CurrS]
       Sub     EAX, [DWord FillArea2+(Sects-CompareText)]
       Test    EAX, EAX  ; ^ Sector count read from the file
       Jns     ItsPos                   ; Negative?
       Mov     BL, "+"
       Neg     EAX
       Jmp     @1

ItsPos:
       Mov     BL, "-"

@1:
       Mov     [MispSign], BL
       Lea     DI, [MispEnd-1]
       Call    PutSect2
       Cmp     CL, 1                    ; Guess what
       Ret

PutSect:
       Mov     EAX, [CurrS]
       Lea     DI, [SectEnd-1]
       Mov     [Sects], EAX

PutSect2:
       Mov     CX, 9
       Lea     EBX, [10]

LongDivLoop2:
       Xor     EDX, EDX
       Div     EBX
       Add     DL, "0"
       Mov     [DI], DL
       Dec     DI
       Cmp     CX, 7
       Je      Dot2
       Cmp     CX, 3
       Je      Dot2
       Dec     CX
       Jnz     LongDivLoop2
       Ret

Dot2:
       Dec     DI
       Dec     CX
       Dec     CX
       Jmp     LongDivLoop2

StartText      DB "BFTest v1.3 started. Press (almost) any key to quit.", 13,10
               DB "Logging to "
OutputDrv      DB "X"                   ; Save erroneous sectors to this dir
               DB ":\BFErrors.$\"
StrL           =  $-StartText
ErrFile        DB "BFError$."           ; "BFError$.AA@", 0
Chr3	       DB "A"
Chr2	       DB "A"
Chr1	       DB "A"-1
	       DB 0

InfText        DB 13,10, "Read:   bytes      sectors   errors",13,10,13,10,"$"
DirOK          DB 0
ErrorDrive     DB 13,10,"Failed to create temp dir on the specified drive!",13,10,"$"
FullText       DB 13,10, "Disk full.", 13,10, "$"
EnoughNames    DB 13,10, "17576 files created, hope you aren't happy!", 13,10, "$"
FName          DB "BFRead$$.$$$",0
FFound         DB "File not found.", 13,10, "$"

CompareText:
	       DB "This is sector "
CompareL2      =  $-CompareText
SectStart      DB "x.xxx.xxx"
SectEnd        DB " of BFRead$$.$$$. "
Sects	       DD 0
               DB 13,10
CompareEnd:
CompareL       =  CompareEnd-CompareText

Align 4
SaveStart:
	       DB "Pos: "
FillArea:
	       DB "x.xxx.xxx.xxx -"
	       DB " Sector: "
	       DB "x.xxx.xxx"
	       DB " - Misplaced by "
MispSign       DB "?"
Misp	       DB "x.xxx.xxx"
MispEnd        DB " sectors."
               DB 13,10
SaveL	       =  $-SaveStart

FillArea2:
CopyR          DB "(C) Copyright 1996 by Sami Farin <sfarin@ratol.fi>. "
               DB "All Rights Reserved. Freeware."

END Start
